@@ -2,7 +2,7 @@
 
     Source file for extracting email addresses from a header.
 
-    Copyright (C) 1997-1999  Richard Gooch
+    Copyright (C) 1997-2002  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,9 +41,27 @@
     Updated by      Richard Gooch   16-MAR-1999: Supported address format:
   "Real Name\nvalid-address".
 
-    Last updated by Richard Gooch   20-APR-1999: Supported address formats with
+    Updated by      Richard Gooch   20-APR-1999: Supported address formats with
   "Real Name" component with \" in name.
 
+    Updated by      Richard Gooch   4-MAY-1999: Added "^Resent-From:" to the
+  test for relay-from addresses.
+
+    Updated by      Richard Gooch   18-FEB-2002: Supported address format:
+  "Real\nName" <username@email.address>
+
+    Updated by      Richard Gooch   3-APR-2002: Supported address format:
+  username@email.address (Real\n Name [,Organisation])
+
+    Updated by      Richard Gooch   9-JUN-2002: Supported address formats:
+  "Real "Name <username@email.address>
+  "Real "Name@email.address
+  "Real Name""Real Name"@email.address
+
+    Last updated by Richard Gooch   7-JUL-2002: Fixed support for address
+  format: "Real "Name@email.address (non-alpha characters were not supported).
+  Simplified quote processing.
+
 
 */
 #include <stdio.h>
@@ -79,6 +97,7 @@ char *get_next_address (decode_type *info, char error[STRING_LENGTH])
     char *ptr, *addr;
 
     error[0] = '\0';
+    if (info->next_pos >= info->line_length) return (NULL);
     if (info->next_pos == 0)
     {
 	/*  Remove trailing whitespace  */
@@ -87,9 +106,10 @@ char *get_next_address (decode_type *info, char error[STRING_LENGTH])
 	     --ptr) *ptr = '\0';
 	info->line_length = ptr - info->linebuf + 1;
     }
-    if ( info->continuation_line || (info->next_pos > 0) )
+    while ( info->continuation_line || (info->next_pos > 0) )
     {
-	return get_address (info, error);
+	addr = get_address (info, error);
+	if ( addr || (info->next_pos >= info->line_length) ) return (addr);
     }
     /*  Start of a new line  */
     info->continuation_line = FALSE;
@@ -121,11 +141,34 @@ char *get_next_address (decode_type *info, char error[STRING_LENGTH])
 	info->next_pos = info->line_length;
 	return (addr);
     }
+    else if ( info->scan_from_relay &&
+	      (st_nicmp (info->linebuf, "resent-from:", 12) == 0) )
+    {
+	for (addr = info->linebuf + 12; isspace (*addr) && (*addr != '\0');
+	     ++addr);
+	if (*addr == '\0')
+	{	
+	    fprintf (stderr, "No address: \"%s\"\n", info->linebuf);
+	    strcpy (error, "no address");
+	    return (NULL);
+	}
+	for (ptr = addr; !isspace (*ptr) && (*ptr != '\0'); ++ptr);
+	*ptr = '\0';
+	info->next_pos = info->line_length;
+	return (addr);
+    }
     else if ( info->scan_sender && (st_nicmp (info->linebuf, "sender:", 7)
 				    == 0) )
 	info->next_pos = 7;
     else return (NULL);
-    return get_address (info, error);
+    while (TRUE)
+    {
+	addr = get_address (info, error);
+	if (addr) return (addr);
+	if (!info->continuation_line) break;
+	if (info->next_pos >= info->line_length) break;
+    }
+    return (NULL);
 }   /*  End Function get_next_address  */
 
 flag read_line (int fd, decode_type *info, FILE *debug_fp, flag *eof)
@@ -203,11 +246,23 @@ static char *get_address (decode_type *info, char error[STRING_LENGTH])
     char *ptr1, *ptr2, *addr;
 
     error[0] = '\0';
-    if (info->next_pos >= info->line_length) return (NULL);
-    addr = info->linebuf + info->next_pos;
-    /*  Check for trailing ','  */
+    /*  Check for trailing ','. Must do this first to prevent infinite loop  */
     info->continuation_line = (info->linebuf[info->line_length - 1]
 			       == ',') ? TRUE : FALSE;
+    if (info->next_pos >= info->line_length) return (NULL);
+    addr = info->linebuf + info->next_pos;
+    if (info->unclosed_character == '(')
+    {   /*  Find closing ')' and discard everything before  */
+	if ( ( ptr1 = strchr (addr, ')') ) == NULL )
+	{   /*  Support name over 2 lines, but not more  */
+	    fprintf (stderr, "Bad address (missing ')'): \"%s\"\n",
+		     addr);
+	    strcpy (error, "bad address (missing ')')");
+	    return (NULL);
+	}
+	info->unclosed_character = '\0';
+	addr = ptr1 + 1;
+    }
     /*  Skip leading whitespace and ',' characters  */
     while ( ( isspace (*addr) || (*addr == ',') ) && (*addr != '\0' ) ) ++addr;
     if (*addr == '\0')
@@ -216,12 +271,12 @@ static char *get_address (decode_type *info, char error[STRING_LENGTH])
 	return (NULL);
     }
     /*  First handle address formats with a leading '"'  */
-    if (*addr == '\"')
-    {
-	/*  Address formats:
+    if ( (*addr == '\"') || (info->unclosed_character == '\"') )
+    {   /*  Address formats:
 		"Real Name"@email.address
-	        "Real Name" <username@email.address>
-		"Real Name"\n<username@email.address>
+	        "Real Name" valid-address
+		"Real Name"\nvalid-address
+	        "Real\nName" valid-address
 	*/
 	/*  Search for '\"' character, but skip escaped quote  */
 	for (ptr1 = addr + 1; *ptr1 != '\0'; ++ptr1)
@@ -229,17 +284,25 @@ static char *get_address (decode_type *info, char error[STRING_LENGTH])
 	    if ( (ptr1[0] == '\"') && (ptr1[-1] != '\\') ) break;
 	}
 	if (ptr1[0] == '\0')
-	{
-	    /*  Stuffed address  */
-	    fprintf (stderr, "Bad address (missing close quote): \"%s\"\n",
-		     addr);
-	    strcpy (error, "bad address (missing close quote)");
+	{   /*  Missing closing quote. Perhaps on the next line?  */
+	    if (info->unclosed_character == '\"')
+	    {   /*  Support name over 2 lines, but not more  */
+		fprintf (stderr, "Bad address (missing close quote): \"%s\"\n",
+			 addr);
+		strcpy (error, "bad address (missing close quote)");
+		return (NULL);
+	    }
+	    info->continuation_line = TRUE;
+	    info->unclosed_character = '\"';
+	    info->next_pos = info->line_length;
 	    return (NULL);
 	}
+	info->unclosed_character = '\0';
 	if (ptr1[1] == '@')
 	{
 	    /*  Address format:
 		"Real Name"@email.address
+		"Real\nName"@email.address
 	    */
 	    while ( !isspace (*ptr1) && (*ptr1 != '\0') && (*ptr1 != ',') )
 		++ptr1;
@@ -248,44 +311,22 @@ static char *get_address (decode_type *info, char error[STRING_LENGTH])
 	    return (addr);
 	}
 	/*  Address format:
-	        "Real Name" <username@email.address>
-		"Real Name"\n<username@email.address>
+	        "Real Name" valid-address
+		"Real Name"\nvalid-address
+	        "Real\nName" valid-address
 	*/
-	/*  Discard bit in quotes  */
+	/*  Discard bit in quotes, skip whitespace and leave it for later  */
 	for (addr = ptr1 + 1; isspace (*addr) && (*addr != '\0'); ++addr);
-	if (*addr == '\0')
-	{
-	    /*  Address format:
-		    "Real Name"\n<username@email.address>
-	    */
-	    /*  Leave it for later  */
-	    info->continuation_line = TRUE;
-	    info->next_pos = info->line_length;
-	    return (NULL);
-	}
-	if (*addr != '<')
-	{
-	    /*  Stuffed address  */
-	    fprintf (stderr, "Bad address (missing '<'): \"%s\"\n", addr);
-	    strcpy (error, "bad address (missing '<')");
-	    return (NULL);
-	}
-	++addr;
-	if ( ( ptr1 = strchr (addr, '>') ) == NULL )
-	{
-	    /*  Stuffed address  */
-	    fprintf (stderr, "Bad address (missing '>'): \"%s\"\n", addr);
-	    strcpy (error, "bad address (missing '>')");
-	    return (NULL);
-	}
-	/*  Address format:
-	        "Real Name" <username@email.address>
-	*/
-	*ptr1++ = '\0';
-	info->next_pos = ptr1 - info->linebuf;
-	return (addr);
+	info->continuation_line = TRUE;
+	info->next_pos = addr - info->linebuf;
+	return (NULL);
     }
     if ( ( ptr1 = strchr (addr, '<') ) != NULL )
+    {   /*  If a ',' comes before '<', we first have a non-"<>" address  */
+	ptr2 = strchr (addr, ',');
+	if ( ptr2 && (ptr2 < ptr1) ) ptr1 = NULL;
+    }
+    if (ptr1)
     {
 	/*  Address formats:
 	        Real Name <username@email.address>
@@ -305,7 +346,9 @@ static char *get_address (decode_type *info, char error[STRING_LENGTH])
     /*  Address formats:
 	    Real Name\nvalid-address
 	    username (Real Name [,Organisation])
+	    username (Real\n Name [,Organisation])
 	    username@email.address (Real Name [,Organisation])
+	    username@email.address (Real\n Name [,Organisation])
 	    username
 	    username@email.address
     */
@@ -331,19 +374,22 @@ static char *get_address (decode_type *info, char error[STRING_LENGTH])
     if (*ptr1 == '(')
     {
 	/*  Address formats:
+	        username (Real\n Name [,Organisation])
 	        username (Real Name [,Organisation])
 	        username@email.address (Real Name [,Organisation])
+	        username@email.address (Real\n Name [,Organisation])
 	*/
-	if ( ( ptr2 = strchr (ptr1, ')') ) == NULL )
-	{
-	    /*  Stuffed address  */
-	    fprintf (stderr, "Bad address (missing ')'): \"%s\"\n", addr);
-	    strcpy (error, "bad address (missing ')'");
-	    return (NULL);
-	}
+	ptr2 = ptr1 + 1;
 	/*  Go backwards and skip trailing spaces  */
 	*ptr1-- = '\0';
 	while ( isspace (*ptr1) && (ptr1 > addr) ) *ptr1-- = '\0';
+	if ( ( ptr2 = strchr (ptr2, ')') ) == NULL )
+	{   /*  Missing closing ')'. Perhaps on the next line?  */
+	    info->continuation_line = TRUE;
+	    info->unclosed_character = '(';
+	    info->next_pos = info->line_length;
+	    return (addr);
+	}
 	*ptr2++ = '\0';
 	info->next_pos = ptr2 - info->linebuf;
 	return (addr);
@@ -361,13 +407,16 @@ static char *get_address (decode_type *info, char error[STRING_LENGTH])
 /*  Valid address formats:
 
     "Real Name"@email.address
-    "Real Name" <username@email.address>
+    "Real Name" valid-address
     "Real Name"\nvalid-address
+    "Real\nName" valid-address
     Real Name <username@email.address>
     Real Name\nvalid-address
     <username@email.address>
     username@email.address
     username
     username (Real Name [,Organisation])
+    username (Real\n Name [,Organisation])
     username@email.address (Real Name [,Organisation])
+    username@email.address (Real\n Name [,Organisation])
 */
