@@ -2,7 +2,7 @@
 
     Source file for option processing for  PGPsendmail  (wrapper to sendmail).
 
-    Copyright (C) 1994-1998  Richard Gooch
+    Copyright (C) 1994-1999  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -60,7 +60,10 @@
     Updated by      Richard Gooch   9-OCT-1997: Made use of new function
   <st_expr_expand>.
 
-    Last updated by Richard Gooch   23-JAN-1998: Added #include <errno.h>
+    Updated by      Richard Gooch   23-JAN-1998: Added #include <errno.h>
+
+    Last updated by Richard Gooch   8-JUN-1999: Took account of change to
+  <st_expr_expand>.
 
 
 */
@@ -357,6 +360,7 @@ static void log_address (CONST char *address, char *line, FILE *debug_fp)
     char *logfile = line;
     CONST char *strings[2];
     char addr[STRING_LENGTH], filename[STRING_LENGTH];
+    static char function_name[] = "log_address";
 
     /*  Find end of filename  */
     for (ptr = line; !isspace (*ptr) && (*ptr != '\0'); ++ptr);
@@ -368,19 +372,19 @@ static void log_address (CONST char *address, char *line, FILE *debug_fp)
 	/*  Find end of filename  */
 	for (ptr = line; !isspace (*ptr) && (*ptr != '\0'); ++ptr);
 	if (*ptr != '\0') *ptr++ = '\0';
-	st_expr_expand (filename, STRING_LENGTH, line, debug_fp);
+	st_expr_expand (filename, STRING_LENGTH, line, NULL, NULL, debug_fp);
 	line = ptr;
 	strings[0] = address;
 	if ( !search_file (filename, strings, debug_fp, &found) )
 	{
 	    if (debug_fp != NULL)
-		fprintf (debug_fp, "Error searching file: \"%s\"\n",
-			 filename);
+		fprintf (debug_fp, "%s: error searching file: \"%s\"\n",
+			 function_name, filename);
 	    return;
 	}
 	if (found) return;
     }
-    st_expr_expand (filename, STRING_LENGTH, logfile, debug_fp);
+    st_expr_expand (filename, STRING_LENGTH, logfile, NULL, NULL, debug_fp);
     if (debug_fp != NULL)
 	fprintf (debug_fp, "Logging new email address: \"%s\"\n", address);
     if ( ( fd = open (filename, O_WRONLY | O_APPEND, 0) ) == -1 )
