@@ -2,7 +2,7 @@
 
     Source file for  PGPdaemon  (automatic signer/encryptor for PGP).
 
-    Copyright (C) 1994-1999  Richard Gooch
+    Copyright (C) 1994-2002  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -168,9 +168,16 @@
     Updated by      Richard Gooch   10-JUL-1998: Removed definitions of system
   errlist array.
 
-    Last updated by Richard Gooch   13-FEB-1999: Added regular inversion of
+    Updated by      Richard Gooch   13-FEB-1999: Added regular inversion of
   passphrase
 
+    Updated by      Richard Gooch   8-JUN-1999: Took account of change to
+  <st_expr_expand>.
+
+    Updated by      Richard Gooch   18-FEB-2002: Declare main() as int.
+
+    Last updated by Richard Gooch   25-MAR-2002: Added NOMAIL config keyword.
+
 
 */
 #include <stdio.h>
@@ -211,7 +218,7 @@ STATIC_FUNCTION (flag set_lockfile, (char *filename, flag lock) );
 STATIC_FUNCTION (flag set_fd_lock, (int fd, flag lock, flag no_filelocking) );
 STATIC_FUNCTION (flag check_and_process_mail, () );
 STATIC_FUNCTION (void read_config, (char *pgppath, flag *mailwait,
-				    long *interval, flag check_mail) );
+				    long *interval, flag *check_mail) );
 STATIC_FUNCTION (flag write_secring, () );
 STATIC_FUNCTION (flag scrub_secring, () );
 STATIC_FUNCTION (int get_rand, () );
@@ -229,7 +236,6 @@ static flag incoming_locked = FALSE;
 static flag insecure = TRUE;
 static flag dump_secring = FALSE;
 static flag caught_sigusr2 = FALSE;
-static flag check_mail = TRUE;
 static flag no_filelocking = FALSE;
 static flag caught_sigio = FALSE;
 static char incoming_spool[STRING_LENGTH];
@@ -268,14 +274,13 @@ int code;
 
 /*  Now everything else may follow.  */
 
-void main (argc, argv, envp)
-int argc;
-char *argv[];
-char *envp[];
+int main (int argc, char *argv[], char *envp[])
 {
     FILE *fp;
     flag mailwait = FALSE;
     flag login_session = TRUE;
+    flag check_mail = TRUE;
+    flag do_fork = FALSE;
     int ppid;
     int sock, fd;
     int tmp, count;
@@ -296,7 +301,6 @@ char *envp[];
     extern flag incoming_locked;
     extern flag insecure;
     extern flag dump_secring;
-    extern flag check_mail;
     extern flag no_filelocking;
     extern flag caught_sigio;
     extern SECRING_SIZE_TYPE secring_bytes;
@@ -353,6 +357,11 @@ char *envp[];
 		myexit (RV_SYS_ERROR);
 	    }
 	}
+	else if (strcmp (argv[count], "-fork") == 0)
+	{
+	    do_fork = TRUE;
+	    login_session = FALSE;
+	}
 	else
 	{
 	    fprintf (stderr, "%s\n", usage_string);
@@ -429,18 +438,44 @@ char *envp[];
 	myexit (RV_SYS_ERROR);
     }
     hostname[STRING_LENGTH - 1] = '\0';
-    read_config (pgppath, &mailwait, &max_interval, check_mail);
+    read_config (pgppath, &mailwait, &max_interval, &check_mail);
     /*  Worry about communication files  */
     sprintf (pid_filename, "%s/.pgpd.PID.%s", pgppath, hostname);
     sprintf (inpipe_filename, "%s/.pgpd.input.%s", pgppath, hostname);
     sprintf (outpipe_filename, "%s/.pgpd.output.%s", pgppath, hostname);
     if (access (pid_filename, F_OK) == 0)
     {
-	fprintf (stderr, "File: \"%s\" exists:\n", pid_filename);
-	fprintf (stderr, "PGPdaemon did not clean up last time\n");
-	myexit (RV_UNDEF_ERROR);
+	int old_pid;
+
+	if ( ( fp = fopen (pid_filename, "r") ) == NULL )
+	{
+	    fprintf (stderr, "Error opening file: \"%s\"\t%s\n",
+		     pid_filename, ERRSTRING);
+	    myexit (RV_CANNOT_OPEN);
+	}
+	if (fscanf (fp, "%d", &old_pid) != 1)
+	{
+	    fprintf (stderr, "Error reading file: \"%s\"\t%s\n",
+		     pid_filename, ERRSTRING);
+	    myexit (RV_CANNOT_OPEN);
+	}
+	if (kill (old_pid, 0) == 0)
+	{
+	    fprintf (stderr, "File: \"%s\" exists and has valid pid: %d\n",
+		     pid_filename, old_pid);
+	    fprintf (stderr, "PGPdaemon appears to be still running\n");
+	    myexit (RV_UNDEF_ERROR);
+	}
+	/*  Appears to be a stale pidfile: remove it  */
+	if (unlink (pid_filename) != 0)
+	{
+	    fprintf (stderr, "Error unlinking file: \"%s\"\t%s\n",
+		     pid_filename, ERRSTRING);
+	    myexit (RV_CANNOT_OPEN);
+	}
+	fprintf (stderr, "Stale pidfile: \"%s\" removed\n", pid_filename);
     }
-    if (errno != ENOENT)
+    else if (errno != ENOENT)
     {
 	fprintf (stderr, "Error testing for PID file: \"%s\"\t%s\n",
 		 pid_filename, ERRSTRING);
@@ -496,15 +531,6 @@ char *envp[];
 	    myexit (RV_SYS_ERROR);
 	}
     }
-    /*  Write PID  */
-    if ( ( fp = fopen (pid_filename, "w") ) == NULL )
-    {
-	fprintf (stderr, "Error writing PID file: \"%s\"\t%s\n",
-		 pid_filename, ERRSTRING);
-	myexit (RV_CANNOT_OPEN);
-    }
-    fprintf ( fp, "%d\n", getpid () );
-    fclose (fp);
     /*  Open logfile  */
     if ( ( fd = open (logfile,
 		      O_WRONLY | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR) )
@@ -536,6 +562,30 @@ char *envp[];
     {
 	fputs ("Passphrase memory sucessfully locked\n", stderr);
     }
+    if (do_fork)
+    {
+	switch ( fork () )
+	{
+	  case 0:   /*  Child   */
+	    break;
+	  case -1:  /*  Error   */
+	    fprintf (stderr, "Error forking\t%s\n", ERRSTRING);
+	    myexit (RV_CANNOT_OPEN);
+	    /*break;*/
+	  default:  /*  Parent  */
+	    _exit (RV_OK);  /*  No need to clean up: no sensitive info yet  */
+	    /*break;*/
+	}
+    }
+    /*  Write PID  */
+    if ( ( fp = fopen (pid_filename, "w") ) == NULL )
+    {
+	fprintf (stderr, "Error writing PID file: \"%s\"\t%s\n",
+		 pid_filename, ERRSTRING);
+	myexit (RV_CANNOT_OPEN);
+    }
+    fprintf ( fp, "%d\n", getpid () );
+    fclose (fp);
     while (keep_going)
     {
 	invert_passphrase ();
@@ -1506,22 +1556,19 @@ static flag check_and_process_mail ()
     return (TRUE);
 }   /*  End Function check_and_process_mail  */
 
-static void read_config (pgppath, mailwait, interval, check_mail)
-/*  This routine will read the configuration file:  $PGPPATH/PGPdaemon.config
-    The value of the PGPPATH environment variable must be pointed to by
-    pgppath  .
-    If the "MAILWAIT" keyword is present, the value TRUE will be written to the
-    storage pointed to by  mailwait  .
-    If the "MAIL_CHECK_INTERVAL" keyword is present, the associated integer
-    value will be written to the storage pointed to by  interval  .
-    If the value of  check_mail  is TRUE, the routine will check for the
-    writability of the mail spool directory.
-    The routine returns nothing.
+static void read_config (char *pgppath, flag *mailwait, long *interval,
+			 flag *check_mail)
+/*  [SUMMARY] Read the configuration file:  $PGPPATH/PGPdaemon.config
+    <pgppath> The value of the PGPPATH environment variable.
+    <mailwait> TRUE will be written here if the "MAILWAIT" keyword is present
+    in the config file.
+    <interval> The integer value associated with the "MAIL_CHECK_INTERVAL"
+    keyword is written here.
+    <check_mail> If this points to the value TRUE, the routine will check for
+    the writability of the mail spool directory. The "NOMAIL" keyword may also
+    cause FALSE to be written here.
+    [RETURNS] Nothing.
 */
-char *pgppath;
-flag *mailwait;
-long *interval;
-flag check_mail;
 {
     char *keyword, *rest;
     char *tmp;
@@ -1583,6 +1630,7 @@ flag check_mail;
 	    }
 	}
 	else if (strcmp (keyword, "LOG_FILE") == 0) strcpy (logfile, rest);
+	else if (strcmp (keyword, "NOMAIL") == 0) *check_mail = FALSE;
 	else
 	{
 	    fprintf (stderr, "Illegal config line: \"%s\"\n", txt);
@@ -1624,7 +1672,8 @@ flag check_mail;
 	strcat (incoming_spool, "/");
 	strcat ( incoming_spool, getenv ("USER") );
     }
-    st_expr_expand (incoming_spool, STRING_LENGTH, incoming_spool, NULL);
+    st_expr_expand (incoming_spool, STRING_LENGTH, incoming_spool, NULL, NULL,
+		    NULL);
     /*  Extract spool directory  */
     strcpy (txt, incoming_spool);
     if ( ( tmp = strrchr (txt, '/') ) == NULL )
@@ -1633,7 +1682,7 @@ flag check_mail;
 	myexit (RV_BAD_DATA);
     }
     *tmp = '\0';
-    if ( check_mail && (access (txt, W_OK) != 0) )
+    if ( *check_mail && (access (txt, W_OK) != 0) )
     {
 	if (errno == ENOENT)
 	{
@@ -1651,18 +1700,13 @@ flag check_mail;
 	myexit (RV_SYS_ERROR);
     }
     if (outgoing_spool[0] == '\0')
-    {
 	sprintf ( outgoing_spool, "%s/decrypted-mail", getenv ("HOME") );
-    }
     else
-    {
-	st_expr_expand (outgoing_spool, STRING_LENGTH, outgoing_spool, NULL);
-    }
+	st_expr_expand (outgoing_spool, STRING_LENGTH, outgoing_spool,
+			NULL, NULL, NULL);
     if (logfile[0] == '\0')
-    {
 	sprintf (logfile, "%s/PGPdaemon.log.%s", pgppath, hostname);
-    }
-    else st_expr_expand (logfile, STRING_LENGTH, logfile, NULL);
+    else st_expr_expand (logfile, STRING_LENGTH, logfile, NULL, NULL, NULL);
 }   /*  End Function read_config  */
 
 static flag write_secring ()
@@ -1749,8 +1793,6 @@ static void invert_passphrase ()
     int length = strlen (passphrase);
 
     for (count = 0; count < length; ++count)
-    {
 	passphrase[count] = ~passphrase[count];
-    }
     xor_char = ~xor_char;
 }   /*  End Function invert_passphrase  */
