@@ -2,7 +2,7 @@
 
     Source file for  pgppipe  (decryption programme).
 
-    Copyright (C) 1994-1997  Richard Gooch
+    Copyright (C) 1994-2002  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,9 +44,11 @@
     Updated by      Richard Gooch   1-OCT-1997: Added -binary and -force
   options.
 
-    Last updated by Richard Gooch   2-OCT-1997: Fixed binary decrypt: to_fd
+    Updated by      Richard Gooch   2-OCT-1997: Fixed binary decrypt: to_fd
   was not closed hence programme would hang.
 
+    Last updated by Richard Gooch   18-FEB-2002: Declare main() as int.
+
 
 */
 #include <stdio.h>
@@ -69,7 +71,7 @@ STATIC_FUNCTION (flag binary_decrypt, (CONST char *pgppath) );
 
 /*  Public functions follow  */
 
-void main (int argc, char **argv, char **envp)
+int main (int argc, char **argv, char **envp)
 {
     flag binary = FALSE;
     flag force = FALSE;
