@@ -2,7 +2,7 @@
 
     Source file for  PGPsendmail  (wrapper to sendmail).
 
-    Copyright (C) 1994-1998  Richard Gooch
+    Copyright (C) 1994-2008  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -128,9 +128,19 @@
     Updated by      Richard Gooch   15-APR-1998: Support BSD mailer which
   passes "send-mail" as argv[0].
 
-    Last updated by Richard Gooch   10-JUL-1998: Removed definitions of system
+    Updated by      Richard Gooch   10-JUL-1998: Removed definitions of system
   errlist array.
 
+    Updated by      Richard Gooch   18-OCT-1999: Added "SENDMAIL" option.
+  Removed string comparison of real sendmail path and argv[0]: not robust.
+
+    Updated by      Richard Gooch   20-OCT-1999: Added "INP_FILTER" option.
+
+    Updated by      Richard Gooch   18-FEB-2002: Declare main() as int.
+
+    Last updated by Richard Gooch   3-MAY-2008: Add support for _PATH_SENDMAIL
+  variable. Add "pgpsendmail" to the list of sendmail-like programmes.
+
 
 */
 #include <stdio.h>
@@ -166,6 +176,11 @@ static char *username = NULL;
 static char real_sendmail[STRING_LENGTH];
 
 
+/*  Private functions  */
+STATIC_FUNCTION (CONST char *get_variable, (CONST char *variable,void *info) );
+STATIC_FUNCTION (flag get_keyword_value,
+		 (FILE *config_fp, FILE *debug_fp, CONST char *keyword,
+		  char *value) );
 STATIC_FUNCTION (flag process_addresses,
 		 (decode_type *info, FILE *config_fp,
 		  option_type *options, FILE *debug_fp) );
@@ -183,10 +198,7 @@ STATIC_FUNCTION (void include_file, (FILE *config_fp, FILE *debug_fp,
 STATIC_FUNCTION (void wake_pgpdaemon, (CONST char *pgppath, FILE *debug_fp) );
 
 
-void main (argc, argv, envp)
-int argc;
-char *argv[];
-char *envp[];
+int main (int argc, char *argv[], char *envp[])
 {
     int tmp;
 
@@ -195,21 +207,15 @@ char *envp[];
     option_type options;
     flag header = TRUE;
     flag quoted_printables = FALSE;
-    flag continuation_line = FALSE;
     flag dummy;
     int count;
-    int ch;
-    int tmpfd;
-    int sendmail_in_fd, sendmail_pid;
-    int out_fd, err_fd;
-    int pgp_in_fd, pgp_pid;
-    int status;
-    int to_fd, from_fd;
+    int ch, tmpfd, status;
+    int in_fd, out_fd, err_fd, pgp_in_fd, to_fd, from_fd;
+    pid_t sendmail_pid, pgp_pid;
     struct stat statbuf;
     decode_type info;
-    char *bad_recipient;
-    char *ptr1, *ptr2;
-    FILE *config_fp;
+    char *bad_recipient, *ptr;
+    FILE *config_fp = NULL;
     char txt[LINE_LENGTH];
     extern FILE *debug_fp;
     extern char *pgppath;
@@ -229,41 +235,40 @@ char *envp[];
 #else
     sprintf (real_sendmail, "%s.real", _PATH_SENDMAIL);
 #endif
+    if ( ( pgppath = getenv ("PGPPATH") ) != NULL )
+    {
+	/*  Open the configuration file only once (said with a French accent)
+	 */
+	sprintf (txt, "%s/PGPsendmail.config", pgppath);
+	config_fp = fopen (txt, "r");
+    }
+    if ( get_keyword_value (config_fp, stderr, "SENDMAIL", real_sendmail) )
+    {
+	st_expr_expand (real_sendmail, STRING_LENGTH, real_sendmail,
+			get_variable, NULL, stderr);
+    }
     if (access (real_sendmail, X_OK) != 0)
     {
 	fprintf (stderr, "File: \"%s\" is not executable\t%s\n",
 		 real_sendmail, ERRSTRING);
 	exit (RV_SYS_ERROR);
     }
-    /*  Try to trap if real_sendmail references PGPsendmail. I hope this works.
-	I only do this to protect people from themselves: if they *read* the
-	documentation, this wouldn't be necessary. Gosh, I'm such a nice
-	person ;-)  */
-    ptr1 = strrchr (real_sendmail, '/');
-    ptr1 = (ptr1 == NULL) ? real_sendmail : ptr1 + 1;
-    ptr2 = strrchr (argv[0], '/');
-    ptr2 = (ptr2 == NULL) ? argv[0] : ptr2 + 1;
-    if (strcmp (ptr1, ptr2) == 0)
-    {
-	fprintf (stderr,
-		 "PGPsendmail appears to have been installed as: \"%s\"\n",
-		 real_sendmail);
-	fprintf (stderr, "Someone didn't read the installation notes...\n");
-	exit (RV_UNDEF_ERROR);
-    }
-    if (strcmp (ptr2, "mailq") == 0)
+    ptr = strrchr (argv[0], '/');
+    ptr = (ptr == NULL) ? argv[0] : ptr + 1;
+    if (strcmp (ptr, "mailq") == 0)
     {
 	execve (real_sendmail, argv, envp);
 	exit (1);
     }
-    if ( (strcmp (ptr2, "sendmail") != 0) &&
-	 (strcmp (ptr2, "send-mail") != 0) )
+    if ( (strcmp (ptr, "sendmail") != 0)
+	 && (strcmp (ptr, "send-mail") != 0)
+	 && (strcmp (ptr, "pgpsendmail") != 0) )
     {
 	sleep (1);
 	execve (real_sendmail, argv, envp);
 	exit (1);
     }
-    if ( ( pgppath = getenv ("PGPPATH") ) == NULL )
+    if (pgppath == NULL)
     {
 	/*  Delay 1 second in case the installer didn't read the instructions
 	    and real_sendmail actually refers back to me. Without this delay,
@@ -290,14 +295,23 @@ char *envp[];
     sprintf (txt, "%s/PGPsendmail.debug", pgppath);
     if (access (txt, F_OK) == 0)
     {
+	int fd;
+
 	/*  Turn on verbose debugging  */
 	sprintf ( txt, "/tmp/PGPsendmail.debug.%d", getuid () );
-	if ( ( debug_fp = fopen (txt, "w") ) == NULL )
+	if ( ( fd = open (txt, O_CREAT | O_TRUNC | O_WRONLY,
+			  S_IRUSR | S_IWUSR) ) < 0 )
 	{
 	    fprintf (stderr, "Error opening debugging file: \"%s\"\t%s\n",
 		     txt, ERRSTRING);
 	    exit (RV_CANNOT_OPEN);
 	}
+	if ( ( debug_fp = fdopen (fd, "w") ) == NULL )
+	{
+	    fprintf (stderr, "Error opening debugging fd: \"%s\"\t%s\n",
+		     txt, ERRSTRING);
+	    exit (RV_CANNOT_OPEN);
+	}
 	fprintf (debug_fp, "Debug file for PGPsendmail %s opened\n", VERSION);
 	fflush (debug_fp);
     }
@@ -369,9 +383,6 @@ char *envp[];
 	return_badnews ("PGP would hang");
 	exit (RV_SYS_ERROR);
     }
-    /*  Open the configuration file only once (said with a French accent)  */
-    sprintf (txt, "%s/PGPsendmail.config", pgppath);
-    config_fp = fopen (txt, "r");
     /*  Process "GLOBAL" option in config file  */
     if (debug_fp != NULL)
     {
@@ -453,16 +464,16 @@ char *envp[];
 	fprintf (debug_fp, "Starting real sendmail: \"%s\"\n", real_sendmail);
 	fflush (debug_fp);
     }
-    sendmail_in_fd = -1;
+    in_fd = -1;
     out_fd = OUTPUT_FD;
     err_fd = ERROR_FD;
     /*  Prevent fork bombs bringing the system to it's knees by waiting a bit.
 	Fork bombs occur when real_sendmail references PGPsendmail
 	Hopefully this is actually trapped earlier: but I like suspenders *and*
-	belt.  */
+	belt  */
     sleep (1);
-    if ( ( sendmail_pid = spawn_job (real_sendmail, argv, &sendmail_in_fd,
-				     &out_fd, &err_fd) ) < 0 )
+    if ( ( sendmail_pid = spawn_job (real_sendmail, argv, &in_fd, &out_fd,
+				     &err_fd) ) < 0 )
     {
 	return_badnews ("could not spawn real sendmail");
 	exit (RV_UNDEF_ERROR);
@@ -477,9 +488,9 @@ char *envp[];
 	return_badnews ("could not generate keylist");
 	exit (RV_SYS_ERROR);
     }
-    /*  Make stdout go to  sendmail  */
-    dup2 (sendmail_in_fd, OUTPUT_FD);
-    close (sendmail_in_fd);
+    /*  Make stdout go to sendmail  */
+    dup2 (in_fd, OUTPUT_FD);
+    close (in_fd);
     if (debug_fp != NULL)
     {
 	fprintf (debug_fp, "Adding special PGP arguments\n");
@@ -489,6 +500,44 @@ char *envp[];
     add_string (&pgp_args, "-feat", FALSE);
     add_string (&pgp_args, "+batchmode", FALSE);
     add_string (&pgp_args, "+language=en", FALSE);
+    /*  Apply input filter if required  */
+    if ( get_keyword_value (config_fp, debug_fp, "INP_FILTER", txt) )
+    {
+	char programme[STRING_LENGTH];
+	char arg1[STRING_LENGTH], arg2[STRING_LENGTH], arg3[STRING_LENGTH];
+	char arg4[STRING_LENGTH], arg5[STRING_LENGTH], arg6[STRING_LENGTH];
+	char *argv[8];
+
+	count = sscanf (txt, "%s %s %s %s %s %s %s",
+			programme, arg1, arg2, arg3, arg4, arg5, arg6);
+	argv[0] = programme;
+	argv[1] = arg1;
+	argv[2] = arg2;
+	argv[3] = arg3;
+	argv[4] = arg4;
+	argv[5] = arg5;
+	argv[6] = arg6;
+	argv[7] = NULL;
+	for (count = 1; count < 7; ++count)
+	    st_expr_expand (argv[count], STRING_LENGTH, argv[count],
+			    get_variable, NULL, debug_fp);
+	in_fd = INPUT_FD;
+	out_fd = -1;
+	err_fd = ERROR_FD;
+	if (spawn_job (programme, argv, &in_fd, &out_fd, &err_fd) < 0)
+	{
+	    return_badnews ("could not spawn filter");
+	    exit (RV_UNDEF_ERROR);
+	}
+	if (debug_fp != NULL)
+	{
+	    fprintf (debug_fp, "Started input filter: \"%s\"\n", txt);
+	    fflush (debug_fp);
+	}
+	/*  Make stdin come from filter  */
+	dup2 (out_fd, INPUT_FD);
+	close (out_fd);
+    }
     /*  Process header  */
     if (debug_fp != NULL)
     {
@@ -792,6 +841,60 @@ char *envp[];
     exit (RV_OK);
 }   /*  End Function main  */
 
+
+/*  Private functions follow  */
+
+static CONST char *get_variable (CONST char *variable, void *info)
+{
+
+    if (strcmp (variable, "_PATH_SENDMAIL") == 0) return (_PATH_SENDMAIL);
+    return (NULL);
+}   /*  End Function get_variable  */
+
+static flag get_keyword_value (FILE *config_fp, FILE *debug_fp,
+			       CONST char *keyword, char *value)
+/*  [SUMMARY] Get he value for a keyword.
+    <config_fp> The configuration file. This may be NULL.
+    <debug_fp> Debugging information will be written here. If this is NULL no
+    debugging information is written.
+    <keyword> The keyword that must be specified in the configuration file.
+    <value> The value is written here. If the keyword is not found, nothing is
+    written here.
+    [RETURNS] TRUE if the keyword was found, else FALSE.
+*/
+{
+    flag eof;
+    int fd, len;
+    char *ptr;
+    char buffer[STRING_LENGTH];
+    static char function_name[] = "get_keyword_value";
+
+    if (config_fp == NULL) return (FALSE);
+    if (fseek (config_fp, 0, SEEK_SET) == -1)
+    {
+	if (debug_fp == NULL) return (FALSE);
+	fprintf (debug_fp, "%s(%s): error rewinding config file\t%s\n",
+		 function_name, keyword, ERRSTRING);
+	return (FALSE);
+    }
+    len = strlen (keyword);
+    while (TRUE)
+    {
+	if ( !read_fp_line (config_fp, buffer, STRING_LENGTH, debug_fp, &eof) )
+	{
+	    if (debug_fp != NULL)
+		fprintf (debug_fp,"%s(%s): error reading configuration file\n",
+			 function_name, keyword);
+	    return (FALSE);
+	}
+	if (eof) return (FALSE);
+	if (strncmp (buffer, keyword, len) == 0) break;
+    }
+    for (ptr = buffer + len; isspace (*ptr) && (*ptr != '\0'); ++ptr);
+    strcpy (value, ptr);
+    return (TRUE);
+}   /*  End Function get_keyword_value  */
+
 static flag process_addresses (decode_type *info, FILE *config_fp,
 			       option_type *options, FILE *debug_fp)
 /*  [SUMMARY] Process a line of addresses, adding to the list of recipients.
@@ -1164,49 +1267,28 @@ static void include_file (FILE *config_fp, FILE *debug_fp, CONST char *keyword)
     <config_fp> The configuration file. This may be NULL.
     <debug_fp> Debugging information will be written here. If this is NULL no
     debugging information is written.
-    <keyword> The keyword that must be specified in the configuration file.
+    <keyword> The keyword that must be specified in the configuration file. The
+    value for the keyword provides the filename to include.
     [RETURNS] Nothing.
 */
 {
     flag eof;
     int fd, len;
     char *ptr;
-    char buffer[STRING_LENGTH];
+    char filename[STRING_LENGTH];
     static char function_name[] = "include_file";
 
-    if (config_fp == NULL) return;
-    if (fseek (config_fp, 0, SEEK_SET) == -1)
+    if ( !get_keyword_value (config_fp, debug_fp, keyword, filename) ) return;
+    if ( ( fd = open (filename, O_RDONLY, 0) ) == -1 )
     {
-	if (debug_fp == NULL) return;
-	fprintf (debug_fp, "%s: error rewinding config file\t%s\n",
-		 function_name, ERRSTRING);
+	if (debug_fp != NULL)
+	    fprintf (debug_fp, "Error opening file: \"%s\"\t%s\n",
+		     filename, ERRSTRING);
 	return;
     }
-    len = strlen (keyword);
-    while (TRUE)
-    {
-	if ( !read_fp_line (config_fp, buffer, STRING_LENGTH, debug_fp, &eof) )
-	{
-	    if (debug_fp != NULL)
-		fprintf (debug_fp,
-			 "%s: error reading configuration file\t%s\n",
-			 function_name, ERRSTRING);
-	    return;
-	}
-	if (eof) return;
-	if (strncmp (buffer, keyword, len) != 0) continue;
-	for (ptr = buffer + len; isspace (*ptr) && (*ptr != '\0'); ++ptr);
-	if ( ( fd = open (ptr, O_RDONLY, 0) ) == -1 )
-	{
-	    if (debug_fp != NULL)
-		fprintf (debug_fp, "Error opening file: \"%s\"\t%s\n",
-			 ptr, ERRSTRING);
-	    return;
-	}
-	fflush (stdout);
-	copy_data (OUTPUT_FD, fd, FALSE);
-	close (fd);
-    }
+    fflush (stdout);
+    copy_data (OUTPUT_FD, fd, FALSE);
+    close (fd);
 }   /*  End Function include_file  */
 
 static void wake_pgpdaemon (CONST char *pgppath, FILE *debug_fp)
