@@ -1,8 +1,8 @@
-/*  pgppipe.c
+/*  pgpsign.c
 
-    Source file for  pgppipe  (decryption programme).
+    Source file for  pgpsign  (signing programme).
 
-    Copyright (C) 1994-1998  Richard Gooch
+    Copyright (C) 1994-2008  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,9 +23,9 @@
       Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
 */
 
-/*  This programme will decrypt messages piped into it and will send the
-    decrypted message to it's standard output. It uses PGPdaemon to perform
-    the decryption.
+/*  This programme will sign messages piped into it and will send the
+    signed message to it's standard output. It uses PGPdaemon to perform
+    the signing.
 
 
     Written by      Richard Gooch   14-JUL-1994: Original pgppipe code.
@@ -36,7 +36,11 @@
     Updated by      Richard Gooch   30-SEP-1997: Various cleanups. Inclusion in
   PGPsendmail/Unix suite.
 
-    Last updated by Richard Gooch   23-JAN-1998: Added #include <errno.h>
+    Updated by      Richard Gooch   23-JAN-1998: Added #include <errno.h>
+
+    Updated by      Richard Gooch   18-FEB-2002: Declare main() as int.
+
+    Last updated by Richard Gooch   10-MAY-2008: Fixed comments.
 
 
 */
@@ -53,10 +57,7 @@
 #include <version.h>
 
 
-void main (argc, argv, envp)
-int argc;
-char *argv[];
-char *envp[];
+int main (int argc, char *argv[], char *envp[])
 {
     flag bool_val;
     int to_fd, from_fd;
